import re
from typing import Callable, Dict, List, Type, Tuple
from flask import current_app
from apispec import BasePlugin, yaml_utils
from flask_classful import FlaskView


# from flask-restplus
RE_URL = re.compile(r"<(?:[^:<>]+:)?([^<>]+)>")


class FlaskClassfulPlugin(BasePlugin):
    """APISpec plugin for Flask classful"""

    @staticmethod
    def flaskpath2openapi(path):
        """Convert a Flask URL rule to an OpenAPI-compliant path.

        :param str path: Flask path template.
        """

        return RE_URL.sub(r"{\1}", path)

    def path_helper(self, operations: Dict, *, view: Type[FlaskView], view_member: Tuple, app=None, **kwargs) -> str:
        """Path helper that allows parsing a Flask classful method."""
        if app is None:
            app = current_app
        name, value = view_member
        endpoint = view.build_route_name(name)

        if hasattr(value, "_rule_cache") and name in value._rule_cache:
            for cached_rule in value._rule_cache[name]:
                rule, options = cached_rule
                sub, ep, options = view.parse_options(options)
                endpoint = ep or endpoint
                rule = app.url_map._rules_by_endpoint[endpoint][0]
                methods = options.get('methods', ['GET'])

                self._load_docstring(value, operations, methods)
                return self.flaskpath2openapi(str(rule))

        if name in view.special_methods:
            methods = view.special_methods[name]
            rule = app.url_map._rules_by_endpoint[endpoint][0]

            self._load_docstring(value, operations, methods)
            return self.flaskpath2openapi(str(rule))

        methods = getattr(view, 'default_methods', ["GET"])
        rule = app.url_map._rules_by_endpoint[endpoint][0]
        self._load_docstring(value, operations, methods)

        return self.flaskpath2openapi(str(rule))

    def _load_docstring(self, view_member: Callable, operations: Dict, methods: List) -> None:
        for method in methods:
            method_name = method.lower()
            operations[method_name] = yaml_utils.load_yaml_from_docstring(
                view_member.__doc__
            )



from setuptools import setup, find_packages

VERSION = "0.1.3"
EXTRAS_REQUIRE = {
    "tests": ["pytest", "mock"],
    "lint": ["flake8==3.9.2", "flake8-bugbear==21.4.3"],
}
EXTRAS_REQUIRE["dev"] = EXTRAS_REQUIRE["tests"] + EXTRAS_REQUIRE["lint"] + ["tox"]


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content


setup(
    name="flask-classful-apispec",
    version=VERSION,
    description="Auto docs generation from marshmallow schema for flask classfy",
    long_description=read("README.rst"),
    author="Rijan adhikari",
    author_email="rijanadhikari@gmail.com",
    url="https://github.com/dev-rijan/flask-classful-apispec",
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=["apispec==5.1.1", "flask-classful==0.14.2"],
    python_requires=">=3.6",
    extras_require=EXTRAS_REQUIRE,
    license="MIT",
    zip_safe=False,
    keywords=[
        "flask-classfull",
        "flask-classful-swagger",
        "apispec",
        "swagger",
        "openapi",
        "specification",
        "documentation",
        "spec",
        "rest",
        "api",
        "web",
        "flask",
        "frameworks",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    test_suite="tests",
    project_urls={
        "Issues": "https://github.com/dev-rijan/flask-classful-apispec/issues",
    },
)



from copy import deepcopy
from apispec import APISpec as BaseAPISpec, yaml_utils
from apispec.core import Components
from apispec.utils import OpenAPIVersion
from collections import OrderedDict
from apispec.ext.marshmallow import MarshmallowPlugin
from flask_classful import FlaskView, get_interesting_members
from flask_classful_apispec.flask_classful_plugin import FlaskClassfulPlugin


class APISpec(BaseAPISpec):
    def __init__(self, app=None, plugins=(), **options):
        if app is not None:
            self.init_app(app, plugins=plugins, **options)

    def init_app(self, app, **options):
        self.title = app.config['DOC_TITLE']
        self.version = app.config['DOC_VERSION']
        self.openapi_version = OpenAPIVersion(app.config['DOC_OPEN_API_VERSION'])
        self.options = options
        self.plugins = (MarshmallowPlugin(), FlaskClassfulPlugin())

        # Metadata
        self._tags = []
        self._paths = OrderedDict()

        # Components
        self.components = Components(self.plugins, self.openapi_version)

        # Plugins
        for plugin in self.plugins:
            plugin.init_spec(self)

    def paths(self, view, app, operations=None):
        members = get_interesting_members(FlaskView, view)
        operations = deepcopy(operations) or OrderedDict()
        operations.update(yaml_utils.load_operations_from_docstring(view.__doc__))

        for member in members:
            self.path(operations=operations, view=view, view_member=member)
